# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::MarketplaceOrdering::Mgmt::V2015_06_01
  #
  # REST API for MarketplaceOrdering Agreements.
  #
  class MarketplaceAgreements
    include MsRestAzure

    #
    # Creates and initializes a new instance of the MarketplaceAgreements class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [MarketplaceOrderingAgreements] reference to the MarketplaceOrderingAgreements
    attr_reader :client

    #
    # Get marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [AgreementTerms] operation results.
    #
    def get(publisher_id, offer_id, plan_id, custom_headers:nil)
      response = get_async(publisher_id, offer_id, plan_id, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(publisher_id, offer_id, plan_id, custom_headers:nil)
      get_async(publisher_id, offer_id, plan_id, custom_headers:custom_headers).value!
    end

    #
    # Get marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(publisher_id, offer_id, plan_id, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      offer_type = 'virtualmachine'
      fail ArgumentError, 'publisher_id is nil' if publisher_id.nil?
      fail ArgumentError, 'offer_id is nil' if offer_id.nil?
      fail ArgumentError, 'plan_id is nil' if plan_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.MarketplaceOrdering/offerTypes/{offerType}/publishers/{publisherId}/offers/{offerId}/plans/{planId}/agreements/current'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'offerType' => offer_type,'publisherId' => publisher_id,'offerId' => offer_id,'planId' => plan_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MarketplaceOrdering::Mgmt::V2015_06_01::Models::AgreementTerms.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Save marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param parameters [AgreementTerms] Parameters supplied to the Create
    # Marketplace Terms operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [AgreementTerms] operation results.
    #
    def create(publisher_id, offer_id, plan_id, parameters, custom_headers:nil)
      response = create_async(publisher_id, offer_id, plan_id, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Save marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param parameters [AgreementTerms] Parameters supplied to the Create
    # Marketplace Terms operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_with_http_info(publisher_id, offer_id, plan_id, parameters, custom_headers:nil)
      create_async(publisher_id, offer_id, plan_id, parameters, custom_headers:custom_headers).value!
    end

    #
    # Save marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param parameters [AgreementTerms] Parameters supplied to the Create
    # Marketplace Terms operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_async(publisher_id, offer_id, plan_id, parameters, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      offer_type = 'virtualmachine'
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'publisher_id is nil' if publisher_id.nil?
      fail ArgumentError, 'offer_id is nil' if offer_id.nil?
      fail ArgumentError, 'plan_id is nil' if plan_id.nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::MarketplaceOrdering::Mgmt::V2015_06_01::Models::AgreementTerms.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.MarketplaceOrdering/offerTypes/{offerType}/publishers/{publisherId}/offers/{offerId}/plans/{planId}/agreements/current'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'offerType' => offer_type,'subscriptionId' => @client.subscription_id,'publisherId' => publisher_id,'offerId' => offer_id,'planId' => plan_id},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MarketplaceOrdering::Mgmt::V2015_06_01::Models::AgreementTerms.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Sign marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [AgreementTerms] operation results.
    #
    def sign(publisher_id, offer_id, plan_id, custom_headers:nil)
      response = sign_async(publisher_id, offer_id, plan_id, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Sign marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def sign_with_http_info(publisher_id, offer_id, plan_id, custom_headers:nil)
      sign_async(publisher_id, offer_id, plan_id, custom_headers:custom_headers).value!
    end

    #
    # Sign marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def sign_async(publisher_id, offer_id, plan_id, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'publisher_id is nil' if publisher_id.nil?
      fail ArgumentError, 'offer_id is nil' if offer_id.nil?
      fail ArgumentError, 'plan_id is nil' if plan_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.MarketplaceOrdering/agreements/{publisherId}/offers/{offerId}/plans/{planId}/sign'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'publisherId' => publisher_id,'offerId' => offer_id,'planId' => plan_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MarketplaceOrdering::Mgmt::V2015_06_01::Models::AgreementTerms.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Cancel marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [AgreementTerms] operation results.
    #
    def cancel(publisher_id, offer_id, plan_id, custom_headers:nil)
      response = cancel_async(publisher_id, offer_id, plan_id, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Cancel marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def cancel_with_http_info(publisher_id, offer_id, plan_id, custom_headers:nil)
      cancel_async(publisher_id, offer_id, plan_id, custom_headers:custom_headers).value!
    end

    #
    # Cancel marketplace terms.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def cancel_async(publisher_id, offer_id, plan_id, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'publisher_id is nil' if publisher_id.nil?
      fail ArgumentError, 'offer_id is nil' if offer_id.nil?
      fail ArgumentError, 'plan_id is nil' if plan_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.MarketplaceOrdering/agreements/{publisherId}/offers/{offerId}/plans/{planId}/cancel'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'publisherId' => publisher_id,'offerId' => offer_id,'planId' => plan_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MarketplaceOrdering::Mgmt::V2015_06_01::Models::AgreementTerms.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get marketplace agreement.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [AgreementTerms] operation results.
    #
    def get_agreement(publisher_id, offer_id, plan_id, custom_headers:nil)
      response = get_agreement_async(publisher_id, offer_id, plan_id, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get marketplace agreement.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_agreement_with_http_info(publisher_id, offer_id, plan_id, custom_headers:nil)
      get_agreement_async(publisher_id, offer_id, plan_id, custom_headers:custom_headers).value!
    end

    #
    # Get marketplace agreement.
    #
    # @param publisher_id [String] Publisher identifier string of image being
    # deployed.
    # @param offer_id [String] Offer identifier string of image being deployed.
    # @param plan_id [String] Plan identifier string of image being deployed.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_agreement_async(publisher_id, offer_id, plan_id, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'publisher_id is nil' if publisher_id.nil?
      fail ArgumentError, 'offer_id is nil' if offer_id.nil?
      fail ArgumentError, 'plan_id is nil' if plan_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.MarketplaceOrdering/agreements/{publisherId}/offers/{offerId}/plans/{planId}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'publisherId' => publisher_id,'offerId' => offer_id,'planId' => plan_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::MarketplaceOrdering::Mgmt::V2015_06_01::Models::AgreementTerms.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List marketplace agreements in the subscription.
    #
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array] operation results.
    #
    def list(custom_headers:nil)
      response = list_async(custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # List marketplace agreements in the subscription.
    #
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(custom_headers:nil)
      list_async(custom_headers:custom_headers).value!
    end

    #
    # List marketplace agreements in the subscription.
    #
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.MarketplaceOrdering/agreements'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = {
              client_side_validation: true,
              required: false,
              serialized_name: 'parsed_response',
              type: {
                name: 'Sequence',
                element: {
                    client_side_validation: true,
                    required: false,
                    serialized_name: 'AgreementTermsElementType',
                    type: {
                      name: 'Composite',
                      class_name: 'AgreementTerms'
                    }
                }
              }
            }
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

  end
end
